{ 4I28 DEMO SOFTWARE - TEST PROGRAM FOR 4I27 }
program POS4I28;
uses CRT;

{$R+}    {Range checking }
{$S+}    {Stack checking }
{$I+}    {I/O checking }

{$I 4I28LOW }                 { include low level 4I27 I/O }
{ program simple motion test for 4I27 }
const I28Card = $220;                          { default 4I27 location }

var

Motor0 : motor;
Motor1 : motor;
Motor2 : motor;
Motor3 : motor;
MoveStatus : byte;

procedure InitFilterParameters(motorx : motor);
begin
  with motorx.Filter do
  begin
    KP := 100;                                   { relativly low gain }
    KI := 50;                                    { integrated error }
    KD := 2000;                                  { some value for damping }
    IL := 10;                                    { low integration limit for now }
  end; {with}
  LoadFilterParameters(motorx);
  UpdateFilter(motorx);
end;

procedure shutdown;
begin
  DisableHBridge(Motor0);
  DisableHBridge(motor1);
  DisableHBridge(Motor2);
  DisableHBridge(motor3);
  TurnOffMotor(Motor0);
  TurnOffMotor(Motor1);
  TurnOffMotor(Motor2);
  TurnOffMotor(Motor3);
  Halt(0);
end;

procedure MoveOK(status : byte);
begin
  if status and ExcessivePosMask <> 0 then
  begin
    writeln('Moves stopped do to excessive position error');
    ShutDown;
  end;
end;

procedure Setup;
begin                                          {main}
  DirectVideo := false;
  writeln('About to reset the 4I28');
  if not Reset4I28(I28Card) then
  begin
    writeln('Couldn''t reset 4I28 card... Bail-out!');
    Shutdown;
  end
  else
  begin
    writeln('LM629''S sucessfully reset...');
    writeln('Proceeding with set-up');
  end;
  AssignMotor(Motor0,I28Card,0);               { assign motor 0 }
  AssignMotor(Motor1,I28Card,1);               { assign motor 1 }
  AssignMotor(Motor2,I28Card,2);               { assign motor 2 }
  AssignMotor(Motor3,I28Card,3);               { assign motor 3 }
  SetMotionMode(Motor0,PositionMode);          { set up motor 0 for position mode }
  SetMotionMode(Motor1,PositionMode);          { set up motor 1 for position mode }
  SetMotionMode(Motor2,PositionMode);          { set up motor 2 for position mode }
  SetMotionMode(Motor3,PositionMode);          { set up motor 3 for position mode }
  SetSamplingInterval(Motor0,0);               { set motor 0 for minimum sampling interval }
  SetSamplingInterval(Motor1,0);               { set motor 1 for minimum sampling interval }
  SetSamplingInterval(Motor2,0);               { set motor 2 for minimum sampling interval }
  SetSamplingInterval(Motor3,0);               { set motor 3 for minimum sampling interval }
  InitFilterParameters(Motor0);
  InitFilterParameters(Motor1);
  InitFilterParameters(Motor2);
  InitFilterParameters(Motor3);
  TurnOffMotor(Motor0);
  TurnOffMotor(Motor1);
  TurnOffMotor(Motor2);
  TurnOffMotor(Motor3);
  LoadPosErrForStop(Motor0,50000);
  LoadPosErrForStop(Motor1,50000);
  LoadPosErrForStop(Motor2,50000);
  LoadPosErrForStop(Motor3,50000);
end;

procedure DoMoves;
begin
  EnableHbridge(Motor0);
  EnableHBridge(Motor1);
  EnableHbridge(Motor2);
  EnableHBridge(Motor3);
  DefineHome(Motor0);
  DefineHome(Motor1);
  DefineHome(Motor2);
  DefineHome(Motor3);
  MoveStatus := 0;
  MoveOK(PolledXYMove(Motor0,Motor1,2000,2000,10000,1000));
  MoveOK(PolledXYMove(Motor0,Motor1,0,0,1000000,1000));
  MoveOK(PolledXYMove(Motor0,Motor1,8000,8000,100000,1000));
  MoveOK(PolledXYMove(Motor0,Motor1,1000,1000,300000,1000));
  MoveOK(PolledXYMove(Motor0,Motor1,-1000,-1000,300000,1000));
  MoveOK(PolledXYMove(Motor0,Motor1,0,0,800000,1500));
  MoveOK(PolledXYMove(Motor2,Motor3,2000,2000,10000,1000));
  MoveOK(PolledXYMove(Motor2,Motor3,0,0,1000000,1000));
  MoveOK(PolledXYMove(Motor2,Motor3,8000,8000,100000,1000));
  MoveOK(PolledXYMove(Motor2,Motor3,20000,20000,300000,1000));
  MoveOK(PolledXYMove(Motor2,Motor3,0,0,800000,1500));
  TurnOffMotor(Motor0);
  TurnOffMotor(Motor1);
  TurnOffMotor(Motor2);
  TurnOffMotor(Motor3);
  DisableHbridge(Motor0);
  DisableHBridge(Motor1);
  DisableHbridge(Motor2);
  DisableHBridge(Motor3);
  MoveStatus := 0;
  SelectY(Motor0);
  SelectY(Motor1);
  SelectY(Motor2);
  SelectY(Motor3);
  EnableHbridge(Motor0);
  EnableHBridge(Motor1);
  EnableHbridge(Motor2);
  EnableHBridge(Motor3);
  DefineHome(Motor0);
  DefineHome(Motor1);
  DefineHome(Motor2);
  DefineHome(Motor3);
  MoveStatus := 0;
  MoveOK(PolledXYMove(Motor0,Motor1,2000,2000,10000,1000));
  MoveOK(PolledXYMove(Motor0,Motor1,0,0,1000000,1000));
  MoveOK(PolledXYMove(Motor0,Motor1,8000,8000,100000,1000));
  MoveOK(PolledXYMove(Motor0,Motor1,1000,1000,300000,1000));
  MoveOK(PolledXYMove(Motor0,Motor1,-1000,-1000,300000,1000));
  MoveOK(PolledXYMove(Motor0,Motor1,0,0,800000,1500));
  MoveOK(PolledXYMove(Motor2,Motor3,2000,2000,10000,1000));
  MoveOK(PolledXYMove(Motor2,Motor3,0,0,1000000,1000));
  MoveOK(PolledXYMove(Motor2,Motor3,8000,8000,100000,1000));
  MoveOK(PolledXYMove(Motor2,Motor3,20000,20000,300000,1000));
  MoveOK(PolledXYMove(Motor2,Motor3,0,0,800000,1500));
  TurnOffMotor(Motor0);
  TurnOffMotor(Motor1);
  TurnOffMotor(Motor2);
  TurnOffMotor(Motor3);
  DisableHbridge(Motor0);
  DisableHBridge(Motor1);
  MoveStatus := 0;
end;

begin
  Setup;
  DoMoves;
  halt;
end.
