{ 4I27 DEMO SOFTWARE - FILTER PARAMETER ADJUST PROGRAM FOR 4I27 }

uses graph,crt,dos;
{$R-}                                          { for speed and some obscure bugs }
{$DEFINE MAIN}                                 { to allow 4I27 low compile by itself }
{$I 4I27LOW }                                  { include low level 4I27 I/O }
{ program 4I27Tune }

type point = record
  X : integer;
  Y : integer;
end;

type box = record
 Start : point;
 Size : point;
end;

type title = string[10];

type labeledbox = record
  TitleBox : box;
  ValueBox : box;
  Title : string;
end;

type steptype = record
  StepSize : longint;
  Velocity : longint;
  Acceleration : longint;
end;

type menuindextype =  (KPI,KDI,KII,ILI,SII,STP,VEL,ACC,MTR,TIM);
type menuboxestype = array[KPI..TIM] of labeledbox;
type menutitlestype = array[KPI..TIM] of string[10];
type menuspeed = (DownFast,DownSlow,UpFast,UpSlow);
type timertype =(NullLoop,ReadLoop);
timeperdivtype = array [1..6] of integer;
delayloopstype = array [1..6] of integer;
type hratetype = record
  TimesPerDiv  : timeperdivtype;
  DelayLoops   : delayloopstype;
end;

const
I27Card = $200;                          { default 4I27 location }
MenuTitles : menutitlestype = ('KP','KD','KI','IL','SI','STEP SIZE','VELOCITY','ACCEL.','MOTOR','TIME / DIV');
FirstMenuIndex = KPI;
LastMenuIndex = TIM;
Times : timeperdivtype = (5,10,20,50,100,200);
SamplesPerDivision = 20;

var
Motors : array[0..1] of motor;
StepParameters : array[0..1] of steptype;
WhichMotor : word;
ScreenBox : labeledbox;
MainBox : labeledbox;
VariablesBox : labeledbox;
ParameterBox : labeledbox;
GraphBox : labeledbox;
MessageBox : labeledbox;

MenuIndex : menuindextype;
MenuBoxes : menuboxestype;
HRates : hratetype;
GraphDriver : integer;
BoxColor : word;
TitleColor : word;
ValueTextColor : word;
TitleTextColor : word;
ValueColor : word;
TimeDivIndex : word;
ArrowSize : word;
ResponseTable : array[0..1000] of longint;

function GetTimeFor(whatLoop : timertype): real;       { calibrate value for delay loops in MS per loop}
var
startwink,startsecond,startminute,starthour : word;
endwink,endsecond,endminute,endhour : word;
delwink,delsecond,delminute,delhour : word;
loopCounter : word;
nLoops : longint;
nTimes : word;
tTime : real;
begin
  case whatLoop of
    NullLoop :
    begin
      nLoops := 100000;
      GetTime(starthour,startminute,startsecond,startwink);
      for nTimes := 1 to 10 do
      begin
        for loopCounter := 1 to nLoops do { nothing };
      end;
      GetTime(endhour,endminute,endsecond,endwink);
    end;{NullLoop case}
    ReadLoop :
    begin
      nLoops := 20000;
      GetTime(starthour,startminute,startsecond,startwink);
      for nTimes := 1 to 10 do
      begin
        for loopCounter := 1 to nLoops do
        begin
          ResponseTable[loopCounter] := ReadRealPosition(Motors[0]);
          if loopCounter = 20 then loopCounter := 20;    { dummy to match data aquisition loop }
        end;{for LoopCounter}
      end;{for nTimes}
      GetTime(endhour,endminute,endsecond,endwink);
    end;{readLoop case}
  end;{case}
  delwink   := (endwink-startwink);
  delsecond := (endsecond-startsecond);
  delminute := (endminute-startminute);
  delhour   := (endhour-starthour);
  if delwink   > 32767 then
    begin
      delwink   := 100 + delwink;
      delsecond := delsecond - 1;
    end;
  if delsecond > 32767 then
   begin
     delsecond := 60 + delsecond;
     delminute := delminute - 1;
   end;
  if delminute > 32767 then
    begin
      delminute := 60 + delminute;
      delhour := delhour - 1;
    end;
  if delhour   > 32767 then delhour   := 24 + delhour;
  tTime := delhour * 360000;                   { got to watch them tricky type casts...}
  tTime := tTime + delminute * 6000;
  tTime := tTime + delsecond * 100;
  tTime := tTime + delwink;
  tTime := tTime /nLoops;
  tTime := tTime /1000;
  GetTimeFor := tTime;
end; { TimePerLoop }

procedure BoxInBox(refBox : labeledbox; var newBox : labeledbox;
                   begX : real; begY : real; endX : real; endY : real);
var
sDelX : integer;
sDelY : integer;
eDelX : integer;
eDelY : integer;
begin
  with refBox.ValueBox do
  begin
    sDelX := round(begX * Size.X);
    sDelY := round(begY * Size.Y);
    eDelX := round((1-endX) * Size.X);
    eDelY := round((1-endY) * Size.Y);
    newBox.ValueBox.Start.X := Start.X + sDelX;
    newBox.ValueBox.Start.Y := Start.Y + sDelY;
    newBox.ValueBox.Size.X := Size.X - (sDelX + eDelX);
    newBox.ValueBox.Size.Y := Size.Y - (sDelY + eDelY);
  end;
end;

procedure FillBox(aBox : box; pattern : word; color : word);
begin
  SetFillStyle(pattern,color);
  with aBox do
  Bar(Start.X +1,Start.Y+ 1,Start.X + Size.X-1,Start.y + Size.y-1);
end;

procedure CenterOfBox(theBox : box; var center : point);
begin
  center.X := theBox.Start.X + (theBox.Size.X div 2);
  center.Y := theBox.Start.Y + (theBox.Size.Y div 2);
end;

procedure DrawBox(theBox : box);
begin
  with theBox do
    rectangle(Start.X,Start.Y,Start.X+Size.X,Start.Y+Size.Y);
end;

procedure CenterTextInBox(theBox : box; text : string);
var
center : point;
begin
  CenterOfBox(theBox,center);
  SetTextJustify(CenterText,CenterText);
  OutTextXY(center.X,center.Y+1,text);
end;

procedure WriteTextValue(theBox : labeledBox; value : string);
begin
  FillBox(theBox.ValueBox,SolidFill,ValueColor);
  SetColor(ValueTextColor);
  CenterTextInBox(theBox.ValueBox,value);
end;

procedure ClearValueBox(theBox : labeledBox);
begin
  FillBox(theBox.ValueBox,SolidFill,ValueColor);
end;

procedure WriteTitle(theBox : labeledbox; title : string);
begin
  FillBox(theBox.TitleBox,SolidFill,TitleColor);
  SetColor(TitleTextColor);
  CenterTextInBox(theBox.TitleBox,title);
end;

procedure WriteNumberValue(var theBox : labeledBox; value : longint);
var
textValue : string;
begin
  Str(value,textValue);
  WriteTextValue(theBox,textValue);
end;


procedure MakeLabeledBox(var theBox : labeledBox; theTitle : string);
var
theTextHeight : word;
theMaxTextWidth : word;
textInfo : TextSettingsType;
begin
  GetTextSettings(textInfo);
  theTextHeight := textInfo.CharSize * 8;
  with theBox do
  begin
    TitleBox.Start.X := ValueBox.Start.X;
    TitleBox.Start.Y := ValueBox.Start.Y;
    TitleBox.Size.X := ValueBox.Size.X ;
    TitleBox.Size.Y := theTextHeight + 2;
    ValueBox.Start.Y := ValueBox.Start.Y + TitleBox.Size.Y;
    ValueBox.Size.Y := ValueBox.Size.Y - TitleBox.Size.Y;
  end;
  WriteTitle(theBox,theTitle);
  SetColor(BoxColor);
  DrawBox(theBox.TitleBox);
  DrawBox(theBox.ValueBox);
end;

procedure InitColors;
begin
  if GraphDriver <> HercMono then
  begin
    BoxColor := EGAWhite;
    TitleColor := EGAWhite;
    ValueTextColor := EGAYellow;
    ValueColor := EGABlue;
    TitleTextColor := EGABlack;
  end
  else
  begin
    BoxColor := White;
    TitleColor := White;
    ValueTextColor := White;
    ValueColor := Black;
    TitleTextColor := Black;
  end;
end;

procedure InitMenu;
var
begX,begY,endX,endY,stepY,sizeY : real;
menuIndex : menuIndextype;
begin
  begX := 0;
  endX := 1;
  begY := 0;
  stepY := 0.101;
  sizeY := 0.08;
  for menuIndex := FirstMenuIndex to LastMenuIndex do
  begin
    BoxInBox(VariablesBox,MenuBoxes[menuIndex],begX,begY,endX,begY+sizeY);
    MakeLabeledBox(MenuBoxes[menuIndex],MenuTitles[menuIndex]);
    begY := begY + stepY
  end;
end;

procedure Arrow(x : word; y : word; color : word);
begin
  Setcolor(color);
  Line(x-ArrowSize,y,x,y);
  Line(x-ArrowSize div 4 ,y+ArrowSize div 4 ,x,y);
  Line(x-ArrowSize div 4 ,y-ArrowSize div 4 ,x,y);
end;

procedure ArrowAtBox(theBox : labeledbox; color : word);
var
xArrow,yArrow : word;
begin
  with theBox.ValueBox do
  begin
    xArrow := Start.X-1;
    yArrow := Start.Y;
  end;
  Arrow(xArrow,yArrow,color);
end;

procedure MarkBox(theBox : Labeledbox);
begin
  ArrowAtBox(theBox,ValueTextColor);
end;

procedure UnMarkBox(theBox : Labeledbox);
begin
  ArrowAtBox(theBox,ValueColor);
end;

procedure MenuIndexUp;
begin
  UnMarkBox(MenuBoxes[MenuIndex]);
  if MenuIndex > FirstMenuIndex then MenuIndex := Pred(MenuIndex);
  MarkBox(MenuBoxes[MenuIndex]);
end;

procedure MenuIndexDown;
begin
  UnMarkBox(MenuBoxes[MenuIndex]);
  if MenuIndex < LastMenuIndex then MenuIndex := Succ(MenuIndex);
  MarkBox(MenuBoxes[MenuIndex]);
end;


procedure InitScreen;
var size : point;
var origin : point;
var aBox : box;
begin
  with ScreenBox.ValueBox do
  begin
    Start.X :=0;
    Start.Y :=0;
    Size.X :=GetMaxX;
    Size.Y :=GetMaxY;
    ArrowSize := Size.X div 25;
    end;
  BoxInBox(ScreenBox,MainBox,0.02,0.02,0.98,0.98);
  MakeLabeledbox(MainBox,'4I27 MOTOR TUNING PROGRAM');
  WriteTextValue(MainBox,' ');
  BoxInBox(MainBox,VariablesBox,0.05,0.05,0.2,0.98);
  BoxInBox(MainBox,GraphBox,0.25,0.05,0.95,0.85);
  MakeLabeledBox(GraphBox,'Graph Window');
  BoxInBox(MainBox,MessageBox,0.25,0.9,0.95,0.98);
  MakeLabeledBox(MessageBox,'Message Box');
  WriteTextValue(MessageBox,'Hello world');
  InitMenu;
  MenuIndex := FirstMenuIndex;
  MenuIndexUp;                                 { forces refresh of arrow }
end;

procedure BumpWordParameter(var parameter : word; speed : menuspeed);
var
newparameter : word;
bump : real;
begin
  case speed of
    DownFast : bump := -0.1;
    DownSlow : bump := -0.01;
    UpFast : bump := 0.1;
    UpSlow : bump := 0.01;
  end; {case}
  newparameter := round(parameter * (1 +bump));
  if parameter = newparameter then             { if it did't change... }
  begin
    if bump >0 then parameter := parameter + 1;{ increment by one }
    if (bump < 0)  and (parameter > 0) then parameter := parameter - 1;{ or decrement by one }
  end
  else parameter := newparameter;
  if parameter >$7FFF then parameter := $7FFF;
end;

procedure BumpLongintParameter(var parameter : longint; speed : menuspeed);
var
newparameter : longint;
bump : real;
begin
  case speed of
    DownFast : bump := -0.1;
    DownSlow : bump := -0.01;
    UpFast : bump := 0.1;
    UpSlow : bump := 0.01;
  end; {case}
  newparameter := round(parameter * (1 +bump));
  if parameter = newparameter then             { if it did't change... }
  begin
    if bump >0 then parameter := parameter + 1;{ increment by one }
    if (bump < 0)  and (parameter > 0) then parameter := parameter - 1;{ or decrement by one }
  end
  else parameter := newparameter;
  if parameter >$3FFFFFFF then parameter := $3FFFFFFF;
end;

procedure DisplayTimeDiv;
var
nString : string;
begin
  Str(HRates.TimesPerDiv[TimeDivIndex],nString);
  WriteTextValue(MenuBoxes[TIM],nString + ' MS');
end;

procedure RefreshDisplayNumbers;
begin
  with Motors[WhichMotor],StepParameters[WhichMotor] do
  begin
    WriteNumberValue(MenuBoxes[KPI],Filter.KP);
    WriteNumberValue(MenuBoxes[KDI],Filter.KD);
    WriteNumberValue(MenuBoxes[KII],Filter.KI);
    WriteNumberValue(MenuBoxes[ILI],Filter.IL);
    WriteNumberValue(MenuBoxes[SII],SamplingInterval shr 8);
    WriteNumberValue(MenuBoxes[STP],StepSize);
    WriteNumberValue(MenuBoxes[VEL],Velocity);
    WriteNumberValue(MenuBoxes[ACC],Acceleration);
    WriteNumberValue(MenuBoxes[MTR],WhichMotor);
    DisplayTimeDiv;
  end;
end;

procedure BumpFilterParameters(bump : menuspeed);
begin
  with Motors[WhichMotor].Filter do
  begin
    case MenuIndex of
      KPI :
      begin
        BumpWordParameter(KP,bump);
        LoadKP(Motors[WhichMotor],KP);
        WriteNumberValue(MenuBoxes[MenuIndex],KP);
      end;
      KDI :
      begin
        BumpWordParameter(KD,bump);
        LoadKD(Motors[WhichMotor],KD);
        WriteNumberValue(MenuBoxes[MenuIndex],KD);
      end;
      KII :
      begin
        BumpWordParameter(KI,bump);
        LoadKI(Motors[WhichMotor],KI);
        WriteNumberValue(MenuBoxes[MenuIndex],KI);
      end;
      ILI :
      begin
        BumpWordParameter(IL,bump);
        LoadIL(Motors[WhichMotor],IL);
        WriteNumberValue(MenuBoxes[MenuIndex],IL);
      end;
    end; {case}
    UpdateFilter(Motors[WhichMotor]);
  end; {with}
end;

procedure BumpSamp(bump : menuspeed);
var
anumber : word;
begin
  anumber := swap(Motors[WhichMotor].SamplingInterval) and $00FF;
  case bump of
    DownFast,DownSlow : if anumber >0 then anumber := anumber -1;
    UpFast,UpSlow : if anumber <255 then anumber := anumber +1;
  end;{case}
  SetSamplingInterval(Motors[WhichMotor],anumber);
  LoadKP(Motors[WhichMotor],Motors[WhichMotor].Filter.KP);  { reload existing KP along with new SI }
  UpdateFilter(Motors[WhichMotor]);
  WriteNumberValue(MenuBoxes[SII],anumber);
end;

procedure BumpStep(bump : menuspeed);
begin
  with StepParameters[WhichMotor] do
  begin
    case bump of
      DownSlow,DownFast :
      begin
        case StepSize of
          200 : StepSize := 100;
          500 : StepSize := 200;
          1000 : StepSize := 500;
          2000 : StepSize := 1000;
          5000 : StepSize := 2000;
          10000 : StepSize := 5000;
        end;{case StepSize}
      end;{down}
      UpSlow,UpFast :
      begin
        case StepSize of
          100 : StepSize := 200;
          200 : StepSize := 500;
          500 : StepSize := 1000;
          1000 : StepSize := 2000;
          2000 : StepSize := 5000;
          5000 : StepSize := 10000;
        end;{case StepSize}
      end;{up}
    end;{case bump}
    WriteNumberValue(MenuBoxes[STP],StepSize);
  end;{with}
end;

procedure BumpVel(bump : menuspeed);
begin
  with StepParameters[WhichMotor] do
  begin
    BumpLongintParameter(Velocity,bump);
    LoadVelocity(Motors[WhichMotor],Velocity);
    WriteNumberValue(MenuBoxes[VEL],Velocity);
  end;
end;

procedure BumpAcc(bump : menuspeed);
begin
  with StepParameters[WhichMotor] do
  begin
    BumpLongintParameter(Acceleration,bump);
    TurnOffMotor(Motors[WhichMotor]);
    LoadAcceleration(Motors[WhichMotor],Acceleration);
    TurnOnMotor(Motors[WhichMotor]);
    WriteNumberValue(MenuBoxes[ACC],Acceleration);
  end;
end;

procedure DeSelectMotor(motor : word );
begin
  TurnOffMotor(Motors[motor])
end;

procedure SelectMotor(motor : word );
begin
  TurnOffMotor(Motors[motor]);
  LoadPosition(Motors[motor],0);
  LoadVelocity(Motors[motor],StepParameters[motor].Velocity);
  LoadAcceleration(Motors[motor],StepParameters[Motor].Acceleration);
  TurnOnMotor(Motors[motor]);
  StartTrajectory(Motors[motor]);
end;

procedure BumpMtr(bump : menuspeed);
var
newMotor : word;
begin
  newMotor := WhichMotor;
  case bump of
    DownFast,DownSlow : if WhichMotor >0 then newMotor := WhichMotor -1;
    UpFast,UpSlow : if WhichMotor <1 then newMotor := WhichMotor +1;
  end;{case}
  If newMotor <> WhichMotor then
  begin
    DeSelectMotor(WhichMotor);
    SelectMotor(NewMotor);
    WhichMotor := newMotor;
    RefreshDisplayNumbers;
  end;
end;

procedure UpTimeIndex;
begin
 if TimeDivIndex < 6 then TImeDivIndex := TimeDivIndex + 1;
end;

procedure DownTimeIndex;
begin
  with HRates do
  begin
    if TimeDivIndex > 1 then                   { note: negative or zero DelayLoops indicates that the PC }
    begin                                      { running the program is not fast enough for that sampling rate }
      if DelayLoops[TimeDivIndex-1] >0 then TimeDivIndex := TimeDivIndex -1;
    end;
  end;
end;


procedure BumpTime(bump : menuspeed);
begin
  case bump of
    DownFast,DownSlow : DownTimeIndex;
    UpFast,UpSlow : UpTimeIndex;
  end;{case}
  DisplayTimeDiv;
end;

procedure BumpOtherParameters(bump : menuspeed);
begin
  case MenuIndex of
    SII :  BumpSamp(bump);
    STP :  BumpStep(bump);
    VEL :  BumpVel(bump);
    ACC :  BumpAcc(bump);
    MTR :  BumpMtr(bump);
    TIM :  BumpTime(bump);
  end; {case}
end;

procedure BumpMenuItem(bump : menuspeed);
begin
  case MenuIndex of
    KPI..ILI : BumpFilterParameters(bump);
    SII..TIM : BumpOtherParameters(bump);
  end; {case}
end;

procedure InitFilterParameters(var motorx : motor);
begin
  with motorx.Filter do
  begin
    KP := 100;                                 { relatively low gain }
    KI := 50;                                  { integrated error }
    KD := 1000;                                { some value for damping }
    IL := 10;                                  { low integration limit for now }
  end; {with}
  SetSamplingInterval(motorx,0);
  LoadFilterParameters(motorx);
  UpdateFilter(motorx);
end;

procedure InitStepParameters(var step : steptype);
begin
  with step do
  begin
    StepSize := 1000;
    Velocity := 10000000;                      { infinitely quick! }
    Acceleration := 10000000;                  { infinitely quick! }
    TurnOffMotor(Motors[WhichMotor]);          { because of pre-Jan 1990 bug }
    LoadAcceleration(Motors[WhichMotor],Acceleration);
    TurnOnMotor(Motors[WhichMotor]);
    LoadVelocity(Motors[WhichMotor],Velocity);
  end; {with}
end;

function InitGraphics : boolean;
var
GraphMode : integer;
ErrorCode : integer;
x : integer;
y : integer;
begin
  GraphDriver := Detect;
  InitGraph(GraphDriver,GraphMode,'');
  ErrorCode := GraphResult;
  If ErrorCode <> GrOk then
  begin
    writeln('Undetermined problem with graphics... aborting progran');
    halt(2);
  end;
end;

procedure AcquireStepResponseData;
var
index : word;
lDelay : word;
begin
  LoadPosition(Motors[WhichMotor],StepParameters[WhichMotor].StepSize);
  WriteTextValue(MessageBox,'Acquiring data');
  for index := 1 to GraphBox.ValueBox.Size.X +1 do
  begin
    ResponseTable[index] := ReadRealPosition(Motors[WhichMotor]);
    if index = SamplesPerDivision then StartTrajectory(Motors[WhichMotor]);
    for lDelay := 1 to HRates.DelayLoops[TimeDivIndex] do {nothing};
  end;
  delay(500);
  LoadPosition(Motors[WhichMotor],0);          { return home... }
  StartTrajectory(Motors[WhichMotor]);
  WriteTextValue(MessageBox,'READY');
end;

procedure ShowGrid;
var
index : word;
scaledData : real;
scaleFactor : real;
begin
  scaleFactor := (GraphBox.ValueBox.Size.Y / StepParameters[WhichMotor].StepSize) /1.25;
  Setcolor(TitleColor);
  SetLineStyle(UserBitLn,$8888,NormWidth);
  with GraphBox.ValueBox do
  begin
    SetViewPort(Start.X,Start.Y,Start.X+Size.X,Start.Y+Size.Y,ClipOn);
    index := SamplesPerDivision;
    while index < Size.X do
    begin
        Line(index,1,index,Size.Y-1);
        index := index + SamplesPerDivision;
    end;
  end;
  SetLineStyle(SolidLn,0,NormWidth);
  SetViewPort(0,0,GetMaxX,GetMaxY,ClipOn);
end;



procedure DisplayGraph;
var
index : word;
scaledData : real;
scaleFactor : real;
offset : real;
begin
  with GraphBox.ValueBox, StepParameters[WhichMotor] do
  begin
  scaleFactor := (Size.Y / StepSize) *0.80;
  offset := (Size.Y / 10);
    SetViewPort(Start.X,Start.Y,Start.X+Size.X,Start.Y+Size.Y,ClipOn);
    for index := 1 to Size.X -1 do
    begin
      scaledData := ResponseTable[index] * ScaleFactor;
      scaledData := (Size.Y - round(offset) - scaledData);
      PutPixel(index,round(scaledData),ValueTextColor);
    end;
  end;
  SetViewPort(0,0,GetMaxX,GetMaxY,ClipOn);
end;

procedure StepGraph;
begin
  AcquireStepResponseData;
  DisplayGraph;
end;

procedure ReInitParameters;
begin
  InitFilterParameters(Motors[WhichMotor]);
  InitStepParameters(StepParameters[WhichMotor]);
  RefreshDisplayNumbers;
end;

procedure KeyLoop;
var
keyChar : char;
done : boolean;

begin
  done := false;
  repeat
    KeyChar := ReadKey;
    case KeyChar of
      #0 :
      begin
        KeyChar := ReadKey;
        case KeyChar of
          #75  : BumpMenuItem(DownFast);
          #79  : BumpMenuItem(DownSlow);
          #81  : BumpMenuItem(UpSlow);
          #77  : BumpMenuItem(UpFast);
          #82  : StepGraph;
          #83  : ClearValueBox(GraphBox);
          #119 : ReInitParameters;
          #73  : ShowGrid;
          #72  : MenuIndexUp;
          #80  : MenuIndexDown;
          #45  : done := true;
        end; {function key case}
      end; {any key case}
    end;
  until done = true;
end;


procedure ShutDown;
begin
  TurnOffMotor(Motors[0]);
  TurnOffMotor(Motors[1]);
  DisableHbridge(Motors[0]);
  DisableHbridge(Motors[1]);
  CloseGraph;
end;

procedure Initialize4I27(address : word);
var
nString : string;
begin
  if not Reset4I27(address) then
  begin
    WriteTextValue(MessageBox,'Couldn''t reset 4I27 card... muddle on!');
    delay(1000);
{    CloseGraph;
    halt(2); }
  end
  else WriteTextValue(MessageBox,'LM629''S sucessfully reset... proceeding with set-up');
  delay(200);
  for WhichMotor := 0 to 1 do
  Begin
    AssignMotor(Motors[WhichMotor],I27Card,WhichMotor);    { assign motor to appropriate half of first card }
    SetMotionMode(Motors[WhichMotor],PositionMode);      { set up motor for Position mode }
    InitFilterParameters(Motors[WhichMotor]);
    InitStepParameters(StepParameters[WhichMotor]);
    LoadPosErrForStop(Motors[WhichMotor],15000);
    DefineHome(Motors[WhichMotor]);
    EnableHbridge(Motors[WhichMotor]);
    Str(WhichMotor,nString);
    WriteTextValue(MessageBox,'Motor '+nString+' initialized');
    delay(200);
  end;
  WhichMotor := 0;                             { start out with motor 0 }
  SelectMotor(WhichMotor);
end;

procedure InitializeTiming;
var
nullTime : real;
readTime : real;
cTime : real;
nString : string;
index : word;
dLoops : integer;
begin
  WriteTextValue(MessageBox,'CALIBRATING DELAY LOOPS');
  Hrates.TimesPerDiv := Times;
  nullTime := GetTimeFor(NullLoop);
  readTime := GetTimeFor(ReadLoop);
  for index := 1 to 6 do
  begin
    cTime := (HRates.TimesPerDiv[index]/SamplesPerDivision/1000);
    dLoops := round((cTime-ReadTime)/nullTime);
    HRates.DelayLoops[index] := dLoops;
  end;
  for index := 6 downto 1 do
  begin
    if HRates.DelayLoops[index] > 0 then TimeDivIndex := index; { point to first positive (valid) value }
  end;
end;

{*****************************************************************************}


begin                                          {main}
  if not InitGraphics then
  begin
    writeln('4I27TUNE must be run on a system with graphics capability');
    halt(2);
  end;
  InitColors;
  InitScreen;
  Initialize4I27(I27Card);
  InitializeTiming;
  RefreshDisplayNumbers;
  WriteTextValue(MessageBox,'READY');
  KeyLoop;
  ShutDown;
  writeln('that''s all folks...');
end.
